// ----------------------------------------------------------------------------------------------------------
// REFORGED BLOOM DX9 EDITION BY THE SANDVICH MAKER
// ----------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------
// UI MULTIPARAMETER OPTIONS
// These defines represent various UI options available in the ENB menu. Change them to any of the following
// options to have separate parameters for different times of day and/or locations:

// SINGLE (it's just the one parameter)
// EI (Exterior: Single, Interior: Single)
// DNI / DN_I (Exterior: Day/Night, Interior: Single)
// DNE_DNI (Exterior: Day/Night, Interior: Day/Night)
// TODI / TOD_I (Exterior: Dawn/Sunrise/Day/Dusk/Sunset/Night, Interior: Single)
// TODE_DNI (Exterior: Dawn/Sunrise/Day/Dusk/Sunset/Night, Interior: Day/Night)
// TODE_TODI (Exterior: Dawn/Sunrise/Day/Dusk/Sunset/Night, Interior: Dawn/Sunrise/Day/Dusk/Sunset/Night)

// But avoid setting boolean and integer parameters to anything other than SINGLE and EI to avoid jarring
// transitions between states.
// Also note that the overhead for using a lot of complex interpolators that are required for options like
// the TOD ones is not free, so try to hold back using them too much unless you really need them.
// ----------------------------------------------------------------------------------------------------------
// Prepass
#define UI_USE_THRESHOLD              SINGLE // Boolean parameter
#define UI_THRESHOLD_VALUES           SINGLE
#define UI_TONEMAP_INPUT              SINGLE // Boolean parameter

// Depth
#define UI_SKYFADE                    SINGLE
#define UI_DEPTH_AWARENESS            SINGLE
#define UI_DEPTH_AWARENESS_CLAMP      SINGLE

#define UI_TEXTURE_BLENDWEIGHT1       SINGLE // 256x256
#define UI_TEXTURE_BLENDWEIGHT2       SINGLE // 128x128
#define UI_TEXTURE_BLENDWEIGHT3       SINGLE // 64x64
#define UI_TEXTURE_BLENDWEIGHT4       SINGLE // 32x32
#define UI_TEXTURE_BLENDWEIGHT5       SINGLE // 16x16
#define UI_TEXTURE_BLENDWEIGHT6       SINGLE // 8x8

#define UI_TEXTURE_TINT1              SINGLE // 256x256
#define UI_TEXTURE_TINT2              SINGLE // 128x128
#define UI_TEXTURE_TINT3              SINGLE // 64x64
#define UI_TEXTURE_TINT4              SINGLE // 32x32
#define UI_TEXTURE_TINT5              SINGLE // 16x16
#define UI_TEXTURE_TINT6              SINGLE // 8x8
#define UI_COUNTERBALANCE_TINT        SINGLE

// Postpass
#define UI_INVERSE_TONEMAP_OUTPUT     SINGLE // Boolean parameter
#define UI_MULTIPLIER                 SINGLE
#define UI_POST_TINT                  SINGLE
#define UI_SATURATION                 SINGLE
#define UI_DESATURATION               SINGLE
#define UI_DESATURATION_CURVE         SINGLE
#define UI_GAMMA                      SINGLE
#define UI_LENS_DIRT_AMOUNT           SINGLE



// ----------------------------------------------------------------------------------------------------------
// PRE-PROCESSOR EDITABLES
// ----------------------------------------------------------------------------------------------------------
// the max threshold selectable in the UI
#define UI_MAX_THRESHOLD 1.0
// 0: Hide threshold controls (and as a result disable thresholding)
// 1: Threshold Center + Threshold Softness
// 2: Threshold Min + Threshold Max
#define UI_THRESHOLD_STYLE 1


// whether to have individual filter sizes for each texture
#define INDIVIDUAL_FILTER_SIZES 1
// add texture blend weights
#define TEXTURE_BLEND_WEIGHTS 1
// add texture tints
#define TEXTURE_TINTS 1


// Controls the range of filter sizes you can select in the ENB GUI.
#define UI_MIN_FILTER_SIZE 4
#define UI_MAX_FILTER_SIZE 36


// whether to include lens dirt feature
#define INCLUDE_LENS_DIRT 0
// lens dirt texture
#define LENS_DIRT_TEXTURE "BloomLensMask.png"



// ----------------------------------------------------------------------------------------------------------
// EXTERNAL PARAMETERS
// ----------------------------------------------------------------------------------------------------------
float4 tempF1; //0,1,2,3
float4 tempF2; //5,6,7,8
float4 tempF3; //9,0
//x=Width, y=1/Width, z=ScreenScaleY, w=1/ScreenScaleY
float4 ScreenSize;
//x=generic timer in range 0..1, period of 16777216 ms (4.6 hours), w=frame time elapsed (in seconds)
float4 Timer;
//additional info for computations
float4 TempParameters;
//MaxTexture1, MaxTexture2, BloomBlueShiftAmount, BloomContrast
float4 BloomParameters;
//changes in range 0..1, 0 means that night time, 1 - day time
float ENightDayFactor;
//changes 0 or 1. 0 means that exterior, 1 - interior
float EInteriorFactor;
//changes in range 0..1, 0 means full quality, 1 lowest dynamic quality (0.33, 0.66 are limits for quality levels)
float EAdaptiveQualityFactor;
//fov in degrees
float FieldOfView;
// Weather And Time
float4 WeatherAndTime;



// ----------------------------------------------------------------------------------------------------------
// REFORGED BLOOM BY THE SANDVICH MAKER
// ----------------------------------------------------------------------------------------------------------
#define REFORGED_HLSL_3 1
#define REFORGED_INCLUDE_FILTERS 1
#define HDR_MAX 16384.0


#include "Reforged/common.fxh"


#define PREVIEW_BLOOM_TEXTURES 0



// ----------------------------------------------------------------------------------------------------------
// ENB GUI
// ----------------------------------------------------------------------------------------------------------
#define UI_CALCULATE_CUSTOM_TOD 1
// TOD Values (have to be defined before macros.fxh is included)
#define DAWN 2.0
#define SUNRISE 7.50
#define DAY 13.0
#define SUNSET 18.50
#define DUSK 2.0
#define NIGHT 0.0 // doesn't actually do anything

#include "Reforged/macros.fxh"


#define UI_CATEGORY Credits
UI_SEPARATOR_CUSTOM("Reforged Bloom DX9")
UI_MESSAGE(Credits1, "by The Sandvich Maker")
UI_WHITESPACE(12)


#define UI_CATEGORY Prepass
UI_SEPARATOR
UI_INT(PassTest, "Pass Test", 0, 2, 0)
#if UI_THRESHOLD_STYLE > 0
    UI_BOOL_MULTI(UI_USE_THRESHOLD, UseThreshold, "Use Threshold", false)
    UI_INT(ThresholdStyle, "Threshold Style", 0, 2, 0)
    UI_INT(ThresholdLuma, "Threshold Luma Style", 0, 2, 1)
    #if UI_THRESHOLD_STYLE == 1
        UI_FLOAT_MULTI(UI_THRESHOLD_VALUES, Threshold, "Threshold", 0.0, UI_MAX_THRESHOLD, 0.5)
        UI_FLOAT_MULTI(UI_THRESHOLD_VALUES, ThresholdSmoothness, "Threshold Softness", 0.0, UI_MAX_THRESHOLD, 0.5)
    #else
        UI_FLOAT_MULTI(UI_THRESHOLD_VALUES, ThresholdMin, "Threshold Min", 0.0, UI_MAX_THRESHOLD, 0.25)
        UI_FLOAT_MULTI(UI_THRESHOLD_VALUES, ThresholdMax, "Threshold Max", 0.0, UI_MAX_THRESHOLD, 0.75)
    #endif
#endif
UI_BOOL_MULTI(UI_TONEMAP_INPUT, TonemapInput, "Tonemap Input", false)
UI_BOOL_MULTI(UI_TONEMAP_INPUT, InverseTonemapInput, "Inverse Tonemap Input", false)


UI_WHITESPACE(1)


#define UI_CATEGORY DepthAware
UI_SEPARATOR_CUSTOM("Depth Aware")
UI_FLOAT_MULTI(UI_SKYFADE, SkyFade, "Sky Fade", 0.0, 1.0, 0.0)
UI_FLOAT_MULTI(UI_DEPTH_AWARENESS, DepthAwareness, "Depth Awareness: Amount", 0.0, 1.0, 0.0)
UI_FLOAT_MULTI(UI_DEPTH_AWARENESS_CLAMP, DepthAwarenessClamp, "Depth Awareness: Clamping", 0.0, 1.0, 0.0)


UI_WHITESPACE(2)


#define UI_CATEGORY Filtering
UI_SEPARATOR
UI_INT(GaussianQuality, "Filter Quality", -2, 2, 0)
#if INDIVIDUAL_FILTER_SIZES
    UI_INT(FilterSize1, "Filter Size 256x256", UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE, 8)
    UI_INT(FilterSize2, "Filter Size 128x128", UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE, 8)
    UI_INT(FilterSize3, "Filter Size 64x64", UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE, 8)
    UI_INT(FilterSize4, "Filter Size 32x32", UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE, 8)
    UI_INT(FilterSize5, "Filter Size 16x16", UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE, 8)
    UI_INT(FilterSize6, "Filter Size 8x8", UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE, 8)
#else
    UI_INT(FilterSize, "Filter Size", UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE, 8)
#endif
UI_INT(PostFilter, "Extra Filter", 0, 2, 0)


UI_WHITESPACE(3)


#define UI_CATEGORY Combine
UI_SEPARATOR_CUSTOM("Texture Combine")


UI_BOOL(UseTex1, "Use Texture 256x256", true)
UI_BOOL(UseTex2, "Use Texture 128x128", true)
UI_BOOL(UseTex3, "Use Texture 64x64", true)
UI_BOOL(UseTex4, "Use Texture 32x32", true)
UI_BOOL(UseTex5, "Use Texture 16x16", true)
UI_BOOL(UseTex6, "Use Texture 8x8", false)


#if TEXTURE_BLEND_WEIGHTS
    UI_WHITESPACE(6)

    UI_BOOL(ApplyWeights, "Apply Weights", true)
    UI_FLOAT_MULTI(UI_TEXTURE_BLENDWEIGHT1, TexWeight1, "Blend Weight 256x256", 0.0, 4.0, 1.0)
    UI_FLOAT_MULTI(UI_TEXTURE_BLENDWEIGHT2, TexWeight2, "Blend Weight 128x128", 0.0, 4.0, 1.0)
    UI_FLOAT_MULTI(UI_TEXTURE_BLENDWEIGHT3, TexWeight3, "Blend Weight 64x64", 0.0, 4.0, 1.0)
    UI_FLOAT_MULTI(UI_TEXTURE_BLENDWEIGHT4, TexWeight4, "Blend Weight 32x32", 0.0, 4.0, 1.0)
    UI_FLOAT_MULTI(UI_TEXTURE_BLENDWEIGHT5, TexWeight5, "Blend Weight 16x16", 0.0, 4.0, 1.0)
    UI_FLOAT_MULTI(UI_TEXTURE_BLENDWEIGHT6, TexWeight6, "Blend Weight 8x8", 0.0, 4.0, 1.0)
    UI_BOOL(NormalizeWeights, "Normalize Weights", true)
#endif

#if TEXTURE_TINTS
    UI_WHITESPACE(7)

    UI_BOOL(ApplyTint, "Apply Tint", false)
    UI_FLOAT3_MULTI(UI_TEXTURE_TINT1, Tint1, "Texture Tint 256x256", 1.0, 1.0, 1.0)
    UI_FLOAT3_MULTI(UI_TEXTURE_TINT2, Tint2, "Texture Tint 128x128", 1.0, 1.0, 1.0)
    UI_FLOAT3_MULTI(UI_TEXTURE_TINT3, Tint3, "Texture Tint 64x64", 1.0, 1.0, 1.0)
    UI_FLOAT3_MULTI(UI_TEXTURE_TINT4, Tint4, "Texture Tint 32x32", 1.0, 1.0, 1.0)
    UI_FLOAT3_MULTI(UI_TEXTURE_TINT5, Tint5, "Texture Tint 16x16", 0.84, 0.0, 0.46)
    UI_FLOAT3_MULTI(UI_TEXTURE_TINT6, Tint6, "Texture Tint 8x8", 1.0, 1.0, 1.0)
    UI_BOOL(NormalizeTint, "Normalize Tint", true)
    UI_FLOAT3_MULTI(UI_COUNTERBALANCE_TINT, TintFinal, "Counterbalance Tint", 1.0, 0.968, 0.988)
#endif


UI_WHITESPACE(4)


#define UI_CATEGORY Processing
UI_SEPARATOR
UI_BOOL_MULTI(UI_INVERSE_TONEMAP_OUTPUT, InverseTonemapOutput, "Inverse Tonemap Output", false)
UI_FLOAT_MULTI(UI_MULTIPLIER, Multiplier, "Multiplier", 0.0, 8.0, 1.0)
UI_FLOAT3_MULTI(UI_POST_TINT, PostTint, "Tint", 1.0, 1.0, 1.0)
UI_FLOAT_MULTI(UI_SATURATION, Saturation, "Saturation", 0.0, 8.0, 1.0)
UI_FLOAT_MULTI(UI_DESATURATION, Desaturation, "Desaturation", 0.0, 4.0, 0.0)
UI_FLOAT_MULTI(UI_DESATURATION_CURVE, DesaturationCurve, "Desaturation Curve", 0.1, 2.0, 1.0)
UI_FLOAT_MULTI(UI_GAMMA, Gamma, "Gamma", 0.1, 8.0, 1.0)
#if INCLUDE_LENS_DIRT
    UI_FLOAT_MULTI(UI_LENS_DIRT_AMOUNT, LensDirtAmount, "Lens Dirt", 0.0, 1.0, 0.0)
    UI_FLOAT(LensDirtSaturation, "Lens Dirt Saturation", 0.0, 1.0, 0.0)
#endif



// ----------------------------------------------------------------------------------------------------------
// TEXTURES
// ----------------------------------------------------------------------------------------------------------
TEXTURE_UNIFORM(Original, LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Color,    LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom1,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom2,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom3,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom4,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom5,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom6,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom7,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Bloom8,   LINEAR, Clamp, FALSE)
TEXTURE_UNIFORM(Depth,    LINEAR, Clamp, FALSE)
#if INCLUDE_LENS_DIRT
    TEXTURE_PATH(Lens, LENS_DIRT_TEXTURE, LINEAR, Clamp, FALSE)
#endif



// ----------------------------------------------------------------------------------------------------------
// STRUCTS
// ----------------------------------------------------------------------------------------------------------
struct VS_OUTPUT_BLOOM
{
    float4 vpos : POSITION;
    float2 txcoord0 : TEXCOORD0;
};
struct VS_INPUT_BLOOM
{
    float3 pos : POSITION;
    float2 txcoord0 : TEXCOORD0;
};



// ----------------------------------------------------------------------------------------------------------
// FUNCTIONS
// ----------------------------------------------------------------------------------------------------------
float thresholdLuma(float3 x)
{
    if (ThresholdLuma == 0)
    {
        return dot(x, 0.333);
    }
    else if (ThresholdLuma == 1)
    {
        return max3(x);
    }
    else
    {
        return calculateLuma(x);
    }
}


float sampleDepth(float2 uv)
{
    return saturate(linearDepth(tex2Dlod(SamplerDepth, float4(uv, 0, 0)).x));
}


float4 applyPrepass(float4 res, float2 uv)
{
    float depth = sampleDepth(uv);
    if (InverseTonemapInput) res.xyz = fastReinhardInverse(res.xyz);

    res.xyz *= 1.0 - SkyFade * (depth >= 1.0);

#if UI_THRESHOLD_STYLE > 0
    if (UseThreshold)
    {
    #if UI_THRESHOLD_STYLE == 1
        static const float lowerThreshold = Threshold - ThresholdSmoothness * 0.5;
        static const float upperThreshold = Threshold + ThresholdSmoothness * 0.5;
    #else
        static const float lowerThreshold = ThresholdMin;
        static const float upperThreshold = ThresholdMax;
    #endif
        if (ThresholdStyle == 0)
        {
            res.xyz *= linearstep(lowerThreshold, upperThreshold, thresholdLuma(res.xyz));
        }
        else if (ThresholdStyle == 1)
        {
            res.xyz *= smoothstep(lowerThreshold, upperThreshold, thresholdLuma(res.xyz));
        }
        else
        {
            res.w = max3(res.xyz);
            res.xyz /= max(res.w, 0.001);
            res.w = max(0.0, res.w - lowerThreshold);
            res.xyz *= res.w;
        }
    }
#endif

    if (TonemapInput)
    {
        res.xyz = fastReinhard(res.xyz);
    }

    res.w = depth;
    return clamp(res, 0.0, HDR_MAX);
}


float4 gaussianFilter(sampler2D s, float2 axis, float2 uv, float centerdepth)
{
#if INDIVIDUAL_FILTER_SIZES
    static const float FilterSize =
        TempParameters.w == 1.0 ? FilterSize1 :
        TempParameters.w == 2.0 ? FilterSize2 :
        TempParameters.w == 3.0 ? FilterSize3 :
        TempParameters.w == 4.0 ? FilterSize4 :
        TempParameters.w == 5.0 ? FilterSize5 :
                                  FilterSize6 ;
#endif

    static const float quality = max(0.66, 1.0 + GaussianQuality / 4.0);
    static const float filtersize = round(clamp(FilterSize, UI_MIN_FILTER_SIZE, UI_MAX_FILTER_SIZE) * quality);
    static const float o = -rcp(filtersize * filtersize);

    axis *= TempParameters.z;

    float4 sumColor = 0.0;
    float sumWeight = 0.0;

    for (float i = -filtersize; i <= filtersize; i++)
    {
        float offset = i * 2.0 - 0.5;
        float4 sampleUV = float4(uv + axis * offset, 0, 0);
        offset *= quality;

        float weight = exp(offset * offset * o);
        float4 sampleColor = tex2Dlod(s, sampleUV);

        if (DepthAwareness > 0.0)
        {
            float diff = (sampleColor.w - centerdepth) * DepthAwareness;
            diff = clamp(1.0 - diff, DepthAwarenessClamp, 1.0);
            weight *= diff;
        }

        sumColor += sampleColor * weight;
        sumWeight += weight;
    }
    sumColor /= sumWeight;

    return sumColor;
}


float4 sampleBloom(sampler2D s, float2 uv, float2 pixsize)
{
    if      (PostFilter == 0) return tex2Dlod(s, float4(uv, 0, 0));
    else if (PostFilter == 1) return filter4x4(s, uv, pixsize);
    else                      return filter3x3tent(s, uv, pixsize);
}



// ----------------------------------------------------------------------------------------------------------
// VERTEX SHADER
// ----------------------------------------------------------------------------------------------------------
VS_OUTPUT_BLOOM VS_Bloom(VS_INPUT_BLOOM IN)
{
    VS_OUTPUT_BLOOM OUT;
    OUT.vpos = float4(IN.pos.xyz, 1.0);
    OUT.txcoord0.xy = IN.txcoord0.xy + TempParameters.xy;
    return OUT;
}



// ----------------------------------------------------------------------------------------------------------
// PIXEL SHADERS
// ----------------------------------------------------------------------------------------------------------
float4 PS_BloomPrePass(VS_OUTPUT_BLOOM IN, float2 vPos : VPOS) : COLOR
{
    float2 uv = IN.txcoord0.xy;
    float2 pixsize = TempParameters.z;
    pixsize.y *= ScreenSize.z;

    float4 bloom;
    if (TempParameters.w == 1.0)
    {
        bloom = filterJimenezOptimized(SamplerBloom1, uv, pixsize);
        bloom = applyPrepass(bloom, uv);
    }
    else
    {
        bloom = filter3x3tent(SamplerBloom1, uv, pixsize);
    }

    return bloom;
}


float4 PS_BloomTexture1(VS_OUTPUT_BLOOM In) : COLOR
{
    float2 uv = In.txcoord0.xy;

    float2 pixsize = TempParameters.z; //inversed bloom texture size
    pixsize.y *= ScreenSize.z; //aspect ratio

    float4 srcbloom = tex2Dlod(SamplerBloom1, float4(uv, 0, 0));
    float4 bloom = gaussianFilter(SamplerBloom1, float2(1.0, 0.0), uv, srcbloom.w);

    return bloom;
}


float4 PS_BloomTexture2(VS_OUTPUT_BLOOM In) : COLOR
{
    float2 uv = In.txcoord0.xy;

    float2 pixsize = TempParameters.z; //inversed bloom texture size
    pixsize.y *= ScreenSize.z; //aspect ratio

    float4 srcbloom = tex2Dlod(SamplerBloom1, float4(uv, 0, 0));
    float4 bloom = gaussianFilter(SamplerBloom1, float2(0.0, 1.0), uv, srcbloom.w);

    return bloom;
}


float4 PS_PostPass(VS_OUTPUT_BLOOM In) : COLOR
{
    float4 bloom = 0.0;
    float2 uv = In.txcoord0.xy;

    static const float invTexSize = 1.0 / 512.0;

#if PREVIEW_BLOOM_TEXTURES
    if (uv.y < 0.5) {
        if      (uv.x < 0.25) bloom = sampleBloom(SamplerBloom6, uv, 1.0 * invTexSize);
        else if (uv.x < 0.50) bloom = sampleBloom(SamplerBloom5, uv, 1.0 * invTexSize);
        else if (uv.x < 0.75) bloom = sampleBloom(SamplerBloom1, uv, 2.0 * invTexSize);
        else                  bloom = sampleBloom(SamplerBloom2, uv, 4.0 * invTexSize);
    } else {
        if      (uv.x < 0.25) bloom = sampleBloom(SamplerBloom3, uv, 8.0 * invTexSize);
        else if (uv.x < 0.50) bloom = sampleBloom(SamplerBloom4, uv, 16.0 * invTexSize);
        else if (uv.x < 0.75) bloom = sampleBloom(SamplerBloom7, uv, 32.0 * invTexSize);
        else                  bloom = sampleBloom(SamplerBloom8, uv, 64.0 * invTexSize);
    }
#else
    float3 tex1 = sampleBloom(SamplerBloom1, uv, 2.0  * invTexSize).xyz * UseTex1;
    float3 tex2 = sampleBloom(SamplerBloom2, uv, 4.0  * invTexSize).xyz * UseTex2;
    float3 tex3 = sampleBloom(SamplerBloom3, uv, 8.0  * invTexSize).xyz * UseTex3;
    float3 tex4 = sampleBloom(SamplerBloom4, uv, 16.0 * invTexSize).xyz * UseTex4;
    float3 tex5 = sampleBloom(SamplerBloom7, uv, 32.0 * invTexSize).xyz * UseTex5;
    float3 tex6 = sampleBloom(SamplerBloom8, uv, 64.0 * invTexSize).xyz * UseTex6;

    float3 weight1 = UseTex1;
    float3 weight2 = UseTex2;
    float3 weight3 = UseTex3;
    float3 weight4 = UseTex4;
    float3 weight5 = UseTex5;
    float3 weight6 = UseTex6;

#if TEXTURE_BLEND_WEIGHTS
    if (ApplyWeights)
    {
        tex1 *= TexWeight1;
        tex2 *= TexWeight2;
        tex3 *= TexWeight3;
        tex4 *= TexWeight4;
        tex5 *= TexWeight5;
        tex6 *= TexWeight6;
        if (NormalizeWeights)
        {
            weight1 *= TexWeight1;
            weight2 *= TexWeight2;
            weight3 *= TexWeight3;
            weight4 *= TexWeight4;
            weight5 *= TexWeight5;
            weight6 *= TexWeight6;
        }
    }
#endif

#if TEXTURE_TINTS
    if (ApplyTint)
    {
        tex1 *= Tint1;
        tex2 *= Tint2;
        tex3 *= Tint3;
        tex4 *= Tint4;
        tex5 *= Tint5;
        tex6 *= Tint6;
        if (NormalizeTint)
        {
            weight1 *= Tint1;
            weight2 *= Tint2;
            weight3 *= Tint3;
            weight4 *= Tint4;
            weight5 *= Tint5;
            weight6 *= Tint6;
        }
    }
#endif

    bloom.xyz += tex1 + tex2 + tex3 + tex4 + tex5 + tex6;
    bloom.xyz /= max(0.0001, weight1 + weight2 + weight3 + weight4 + weight5 + weight6);

#if TEXTURE_TINTS
    bloom.xyz *= TintFinal;
#endif

#endif // PREVIEW_BLOOM_TEXTURES

    // CC
    bloom.xyz *= Multiplier;
    bloom.xyz *= (PostTint / (max3(PostTint) + 0.0001));
    bloom.xyz = lerp(calculateLuma(bloom.xyz), bloom.xyz, Saturation);
    bloom.w = max3(bloom.xyz);
    bloom.xyz = bloom.xyz / (bloom.w + 0.0001);
    if (Desaturation > 0.0) bloom.xyz = lerp(bloom.xyz, dot(bloom.xyz, 0.333), (bloom.w * rcp(DesaturationCurve + bloom.w)) * Desaturation);
    if (Gamma != 1.0) bloom.w = pow(bloom.w, Gamma);
    bloom.xyz = bloom.w * bloom.xyz;

    if (InverseTonemapOutput) bloom.xyz = fastReinhardInverse(bloom.xyz);

#if INCLUDE_LENS_DIRT
    float3 dirt = tex2Dlod(SamplerLens, float4(uv, 0, 0)).xyz;
    dirt = lerp(calculateLuma(dirt), dirt, LensDirtSaturation);
    bloom.xyz = lerp(bloom.xyz, bloom.xyz * dirt, LensDirtAmount);
#endif

    return bloom;
}



// ----------------------------------------------------------------------------------------------------------
// TECHNIQUES
// ----------------------------------------------------------------------------------------------------------
technique BloomPrePass
{
    pass p0
    {
        VertexShader = compile vs_3_0 VS_Bloom();
        PixelShader  = compile ps_3_0 PS_BloomPrePass();

        ColorWriteEnable         = ALPHA|RED|GREEN|BLUE;
        CullMode                 = NONE;
        AlphaBlendEnable         = FALSE;
        AlphaTestEnable          = FALSE;
        SEPARATEALPHABLENDENABLE = FALSE;
    }
}

technique BloomTexture1
{
    pass p0
    {
        VertexShader = compile vs_3_0 VS_Bloom();
        PixelShader  = compile ps_3_0 PS_BloomTexture1();

        ColorWriteEnable         = ALPHA|RED|GREEN|BLUE;
        CullMode                 = NONE;
        AlphaBlendEnable         = FALSE;
        AlphaTestEnable          = FALSE;
        SEPARATEALPHABLENDENABLE = FALSE;
    }
}

technique BloomTexture2
{
    pass p0
    {
        VertexShader = compile vs_3_0 VS_Bloom();
        PixelShader  = compile ps_3_0 PS_BloomTexture2();

        ColorWriteEnable         = ALPHA|RED|GREEN|BLUE;
        CullMode                 = NONE;
        AlphaBlendEnable         = FALSE;
        AlphaTestEnable          = FALSE;
        SEPARATEALPHABLENDENABLE = FALSE;
    }
}

technique BloomPostPass
{
    pass p0
    {
        VertexShader = compile vs_3_0 VS_Bloom();
        PixelShader  = compile ps_3_0 PS_PostPass();

        ColorWriteEnable         = ALPHA|RED|GREEN|BLUE;
        CullMode                 = NONE;
        AlphaBlendEnable         = FALSE;
        AlphaTestEnable          = FALSE;
        SEPARATEALPHABLENDENABLE = FALSE;
    }
}



